package cn.iocoder.yudao.module.wealth.dal.mysql.receivable;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBackVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableOrderPayedFeeTypeVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableQueryVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 应收款 Mapper
 *
 * @author 郑屹
 */
@Mapper
public interface ReceivableMapper extends AbstractMapper<ReceivableDO> {
    @Override
    default PageResult<ReceivableDO> selectPage(PageVO page, Object object) {
        if (object instanceof ReceivableQueryVO) {
            ReceivableQueryVO vo = (ReceivableQueryVO) object;
            return selectPage(page, new LambdaQuery<ReceivableDO>()
                    .eqIfPresent(ReceivableDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceivableDO::getReceiptNo, vo.getReceiptNo())
                    .eqIfPresent(ReceivableDO::getOrderId, vo.getOrderId())
                    .eqIfPresent(ReceivableDO::getOrderNo, vo.getOrderNo())
                    .eqIfPresent(ReceivableDO::getTitleEn, vo.getTitleEn())
                    .eqIfPresent(ReceivableDO::getTitleZh, vo.getTitleZh())
                    .eqIfPresent(ReceivableDO::getNum, vo.getNum())
                    .eqIfPresent(ReceivableDO::getWeight, vo.getWeight())
                    .eqIfPresent(ReceivableDO::getVolume, vo.getVolume())
                    .eqIfPresent(ReceivableDO::getFeeType, vo.getFeeType())
                    .eqIfPresent(ReceivableDO::getExchangeRate, vo.getExchangeRate())
                    .eqIfPresent(ReceivableDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceivableDO::getUnitPrice, vo.getUnitPrice())
                    .eqIfPresent(ReceivableDO::getTotalAmount, vo.getTotalAmount())
                    .eqIfPresent(ReceivableDO::getDiscountTotal, vo.getDiscountTotal())
                    .eqIfPresent(ReceivableDO::getDiscountRemark, vo.getDiscountRemark())
                    .eqIfPresent(ReceivableDO::getIsPayAdvance, vo.getIsPayAdvance())
                    .eqIfPresent(ReceivableDO::getState, vo.getState())
                    .eqIfPresent(ReceivableDO::getSource, vo.getSource())
                    .eqIfPresent(ReceivableDO::getAuthor, vo.getAuthor())
                    .betweenIfPresent(ReceivableDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceivableDO::getId));
        }

        return null;
    }

    @Override
    default List<ReceivableDO> selectList(Object object) {
        if (object instanceof ReceivableQueryVO) {
            ReceivableQueryVO vo = (ReceivableQueryVO) object;
            return selectList(new LambdaQuery<ReceivableDO>()
                    .eqIfPresent(ReceivableDO::getReceiptId, vo.getReceiptId())
                    .eqIfPresent(ReceivableDO::getReceiptNo, vo.getReceiptNo())
                    .eqIfPresent(ReceivableDO::getOrderId, vo.getOrderId())
                    .eqIfPresent(ReceivableDO::getOrderNo, vo.getOrderNo())
                    .eqIfPresent(ReceivableDO::getOrderItemId, vo.getOrderItemId())
                    .eqIfPresent(ReceivableDO::getTitleZh, vo.getTitleZh())
                    .eqIfPresent(ReceivableDO::getTitleEn, vo.getTitleEn())
                    .eqIfPresent(ReceivableDO::getNum, vo.getNum())
                    .eqIfPresent(ReceivableDO::getWeight, vo.getWeight())
                    .eqIfPresent(ReceivableDO::getVolume, vo.getVolume())
                    .eqIfPresent(ReceivableDO::getFeeType, vo.getFeeType())
                    .eqIfPresent(ReceivableDO::getFeeSource, vo.getFeeSource())
                    .eqIfPresent(ReceivableDO::getExchangeRate, vo.getExchangeRate())
                    .eqIfPresent(ReceivableDO::getCurrencyId, vo.getCurrencyId())
                    .eqIfPresent(ReceivableDO::getUnitPrice, vo.getUnitPrice())
                    .eqIfPresent(ReceivableDO::getTotalAmount, vo.getTotalAmount())
                    .eqIfPresent(ReceivableDO::getDiscountTotal, vo.getDiscountTotal())
                    .eqIfPresent(ReceivableDO::getDiscountRemark, vo.getDiscountRemark())
                    .eqIfPresent(ReceivableDO::getIsPayAdvance, vo.getIsPayAdvance())
                    .eqIfPresent(ReceivableDO::getState, vo.getState())
                    .eqIfPresent(ReceivableDO::getSource, vo.getSource())
                    .eqIfPresent(ReceivableDO::getAuthor, vo.getAuthor())
                    .betweenIfPresent(ReceivableDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(ReceivableDO::getId));
        }

        return null;
    }

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.to_warehouse_state, ",
            "o.tidan_no, ",
            "o.container_number as container_number, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_warehouse_line de on o.line_id = de.id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0 and r.receipt_id is null ",
            "<when test = 'query.status != null'>",
            "AND o.`status` = #{query.status}",
            "</when>",
            "<when test = 'query.isCargoControl != null'>",
            "AND (o.`is_cargo_control` = #{query.isCargoControl} or o.`is_cargo_control` = #{query.isCargoControl})",
            "</when>",
            "<when test = 'query.customsType != null'>",
            "AND o.`customs_type` = #{query.customsType}",
            "</when>",
            "<when test = 'query.transportId != null'>",
            "AND o.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.isCargoControl != null '>",
            "AND o.`is_cargo_control` = #{query.isCargoControl}",
            "</when>",
            "<when test = 'query.marks != null and query.marks != \"\" '>",
            "AND o.`marks` like concat('%',concat(#{query.marks},'%'))",
            "</when>",
            "<when test = 'query.departureId != null '>",
            "AND de.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null '>",
            "AND de.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",
            "<when test = 'query.title != null and query.title != \"\" '>",
            "AND concat(r.`title_zh`, r.`title_en`) like concat('%',concat(#{query.title},'%'))",
            "</when>",
            "<when test = 'query.consignorNameOrPhone != null and query.consignorNameOrPhone != \"\" '>",
            "AND concat(nor.`name`, nor.`phone`,nor.`company`) like concat('%',concat(#{query.consignorNameOrPhone},'%'))",
            "</when>",
            "<when test = 'query.consigneeNameOrPhone != null and query.consigneeNameOrPhone != \"\" '>",
            "AND concat(nee.`name`, nee.`phone`,nee.`company`) like concat('%',concat(#{query.consigneeNameOrPhone},'%'))",
            "</when>",
            "<when test = 'query.orderNo != null and query.orderNo != \"\" '>",
            "AND r.`order_no` like concat('%',concat(#{query.orderNo},'%'))",
            "</when>",
            "<when test = 'query.tidanNo != null and query.tidanNo != \"\" '>",
            "AND o.`tidan_no` like concat('%',concat(#{query.tidanNo},'%'))",
            "</when>",
            "<when test = 'query.receiptNo != null and query.receiptNo != \"\" '>",
            "AND r.`receipt_no` like concat('%',concat(#{query.receiptNo},'%'))",
            "</when>",
            "<when test = 'query.state != null '>",
            "AND r.`state` #{query.state}",
            "</when>",
            "<when test = 'query.beginCreateTime != null and query.endCreateTime != null '>",
            "AND r.`create_time` between #{query.beginCreateTime} and #{query.endCreateTime}",
            "</when>",
            "<when test = 'query.feeType != null'>",
            "AND r.`fee_type` = #{query.feeType}",
            "</when>",
            "<when test = 'query.customerId != null'>",
            "AND o.`customer_id` = #{query.customerId}",
            "</when>",
            "<when test = 'query.containerNumber != null'>",
            "AND o.`container_number` = #{query.containerNumber}",
            "</when>",
            "</script>"
    })
    IPage<ReceivableBackVO> receivablePage(@Param("mpPage") IPage<ReceivableBackVO> mpPage, @Param("query") ReceivableQueryVO query);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.to_warehouse_state, ",
            "o.tidan_no, ",
            "o.container_number as container_number, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_warehouse_line de on o.line_id = de.id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0 and r.receipt_id is null ",
            "AND o.`order_id` = #{orderId}",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableByOrderId(@Param("orderId") Long orderId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.to_warehouse_state, ",
            "o.tidan_no, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.state = 0 ",
            "AND r.id in ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>",
            "#{id}",
            "</foreach>)",
            "</script>"
    })
    List<ReceivableBackVO> receivableList(List<Long> ids);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "o.to_warehouse_state, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.id =  #{receivableId} ",
            "</script>"
    })
    ReceivableBackVO getReceivableBackVo(@Param("receivableId") Long receivableId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.container_number as container_number, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "ob.objective ->> '$.titleZh' as objective_name_zh, ",
            "ob.objective ->> '$.titleEn' as objective_name_en, ",
            "o.to_warehouse_state, ",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "where r.deleted = 0",
            "AND r.receipt_id =  #{receiptId} ",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableBackVoListByReceiptId(@Param("receiptId") Long receiptId);


    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks, ",
            "o.tidan_no, ",
            "o.`status`, ",
            "o.abnormal_state, ",
            "o.in_warehouse_state, ",
            "o.shipment_state, ",
            "o.transport_id, ",
            "o.channel_id, ",
            "de.departure_id, ",
            "ob.objective_id, ",
            "o.to_warehouse_state, ",
            "er.write_off_proportion as write_off_proportion,",
            "nor.name as consignor_name, ",
            "nor.phone as consignor_phone, ",
            "nee.name as consignee_name, ",
            "nee.phone as consignee_phone",

            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "left join ecw_order_departure de on de.order_id = o.order_id ",
            "left join ecw_order_objective ob on ob.order_id = o.order_id ",
            "left join ecw_order_consignor nor on nor.order_id = o.order_id ",
            "left join ecw_order_consignee nee on nee.order_id = o.order_id ",
            "left join ecw_receipt er on r.receipt_id = er.id ",
            "where r.deleted = 0",
            "AND o.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableBackVO> getReceivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "o.marks ",
            "from ecw_receivable r ",
            "left join ecw_order o on o.order_id = r.order_id",
            "where r.deleted = 0 AND r.is_pay_advance = 1",
            "AND r.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableBackVO> getFirstReceivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableBackVO.class)
    @Select({
            "<script>",
            "select  ",
            "a.collection_type as collection_type, ",
            "a.fee_type as fee_type, ",
            "a.currency_id as currency_id,",
            "c.title_zh as currency,",
            "p.label as collection_type_label,",
            "f.label as fee_type_label,",
            "COALESCE(b.write_off_proportion,0) AS write_off_proportion,",
            "COALESCE (a.discount_total ,0) as discount_total,",
            "a.receipt_id as receipt_id,",
            "IF(write_off_proportion is null,0, a.total_amount * ( IFNULL( write_off_proportion, 0 ) / 100 ))  AS payed_amount, ",
            "a.total_amount as total_amount",
            "from ecw_receivable a  ",
            "left join system_dict_data p on p.dict_type = 'payment_type' and a.collection_type = p.value and p.deleted=0",
            "left join system_dict_data f on f.dict_type = 'receivable_fee_type' and a.fee_type = f.value and f.deleted=0 ",
            "left join ecw_currency c on c.id = a.currency_id",
            "left join ecw_receipt b on a.receipt_id = b.id and b.deleted = 0",
            "where a.deleted = 0 ",
            "and a.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(@Param("orderId") Long orderId);


    @ResultType(ReceivableDO.class)
    @Select({
            "<script>",
            "select r.*, ",
            "IFNULL(u.nickname, '/') as author_name",
            "from ecw_receivable r ",
            "left join system_user u on u.id = r.author",
            "where r.deleted = 0",
            "AND r.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableDO> receivableListByOrderId(@Param("orderId") Long orderId);

    @ResultType(ReceivableDO.class)
    @Select({
            "<script>",
            "select rb.*, ",
            "IFNULL(rt.write_off_proportion, 0) as write_off_proportion",
            "from ecw_receivable rb ",
            "left join ecw_receipt rt on rb.receipt_id = rt.id",
            "where rb.deleted = 0",
            "AND rb.order_id =  #{orderId} ",
            "</script>"
    })
    List<ReceivableDO> allByOrderId(@Param("orderId") Long orderId);

    @Update({
            "update ecw_receivable set `deleted` = 0 ,update_time = now() where `id` = #{receivableId}"
    })
    void revokeDeleteReceivableById(@Param("receivableId") Long receivableId);
}
