package cn.iocoder.yudao.module.wealth.service.receivable;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.apollo.core.event.OrderOperateLogEvent;
import cn.iocoder.yudao.framework.apollo.core.vo.ApplyInfoVO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.wealth.convert.receivable.ReceivableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableMapper;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receivableDiscount.ReceivableDiscountService;
import cn.iocoder.yudao.module.wealth.vo.receipt.ReceiptExportDto;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 应收款 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class ReceivableServiceImpl extends AbstractService<ReceivableMapper, ReceivableDO> implements ReceivableService {

    @Autowired
    private ReceivableMapper receivableMapper;

    @Resource
    @Lazy
    private ReceiptService receiptService;

    @Resource
    private ReceivableDiscountService receivableDiscountService;

    @Resource
    private AdminUserApi adminUserApi;

    @Autowired
    private ApplicationContext applicationContext;

    @Override
    public Long createReceivable(ReceivableCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // TODO 具体业务待完善
//        if (null != createReqVO.getOrderItemId()){
//
//        }
        // 插入
        ReceivableDO receivable = ReceivableConvert.INSTANCE.convert(createReqVO);
        receivable.setCreateTime(new Date());
        receivable.setCreator(creator);
        receivable.setUpdateTime(new Date());
        receivable.setUpdater(creator);
        this.save(receivable);
        // 返回
        return receivable.getId();
    }

    @Override
    public void updateReceivable(ReceivableUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceivableExists(updateReqVO.getId());
        // 更新
        ReceivableDO updateObj = ReceivableConvert.INSTANCE.convert(updateReqVO);
        this.updateById(updateObj);
    }

    @Override
    public void deleteReceivable(Long id) {
        // 校验存在
        ReceivableDO receivableDO = this.getReceivable(id);
        if (receivableDO == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivableDO.getReceiptId() != null) {
            throw exception(RECEIVABLE_REMOVE_BIND_RECEIPT);
        }
        // 删除
        this.removeById(id);
    }

    @Override
    public void deleteReceivableByOrderId(Long orderId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Collection<Integer> feeSourceList) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>()
                .eq(ReceivableDO::getOrderId, orderId)
                .in(CollectionUtil.isNotEmpty(feeSourceList), ReceivableDO::getFeeSource, feeSourceList));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Integer feeSource) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId).eq(ReceivableDO::getFeeSource, feeSource));
    }

    @Override
    public void deleteReceivableByOrderItemId(Long orderItemId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderItemId, orderItemId));
    }

    @Override
    public ReceivableDO getReceivableByOrderItemId(Long orderItemId, Integer feeType) {
        return receivableMapper.selectOne(new LambdaQueryWrapper<ReceivableDO>()
                .eq(ReceivableDO::getOrderItemId, orderItemId)
                .eq(ReceivableDO::getFeeType, feeType)
                .eq(ReceivableDO::getFeeSource, 1));
    }

    @Override
    public void cancelReceivable(Long id) {
        ReceivableDO receivable = this.getReceivable(id);
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        if (receivable.getState() != 0) {
            throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        this.deleteReceivable(id);
    }

    @Override
    public void updateReceivableById(ReceivableDO updateReqVO) {
        if (null == updateReqVO.getId()) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        ReceivableDO receivable = this.getReceivable(updateReqVO.getId());
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        //收款状态（0未收款，1收款中，2已收款）
        if (receivable.getState() != 0) {
            // 取消此逻辑
            //throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        updateReqVO.setUpdateTime(new Date());
        updateReqVO.setUpdater(String.valueOf(SecurityFrameworkUtils.getLoginUserId()));
        // 更新
        this.updateById(updateReqVO);
    }

    @Override
    public void verificationReceivableIsWriteOff(Long orderId, Long orderItemId) {
        if (Objects.nonNull(orderId)) {
            // 校验整个订单的应收单是否有核销的
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderId, orderId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        } else {
            // 校验订单项的应收是否核销
            if (Objects.isNull(orderItemId)) {
                throw exception(ORDER_ITEM_ID_NOT_NULL);
            }
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderItemId, orderItemId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        }
    }

    private void validateReceivableExists(Long id) {
        if (this.getReceivable(id) == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
    }

    @Override
    public ReceivableDO getReceivable(Long id) {
        return this.getById(id);
    }

    @Override
    public List<ReceivableDO> getReceivableList(Collection<Long> ids) {
        return receivableMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceivableDO> getReceivablePage(ReceivableQueryVO query, PageVO page) {
        return receivableMapper.selectPage(page, query);
    }

    @Override
    public PageResult<ReceivableBackVO> receivablePage(ReceivableQueryVO query, PageVO page) {
        query.setLang(I18nMessage.getLang());
        page.setField("r.id");
        IPage<ReceivableBackVO> mpPage = MyBatisUtils.buildPage(page);
        receivableMapper.receivablePage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<ReceivableBackVO> getReceivableByOrderId(Long orderId) {
        return receivableMapper.getReceivableByOrderId(orderId);
    }


    @Override
    public List<ReceivableDO> getReceivableList(ReceivableQueryVO query) {
        return receivableMapper.selectList(query);
    }


    @Override
    public Long createReceipt(Long receivableId) {
        return null;
    }

    @Override
    public List<ReceivableBackVO> receivableList(List<Long> ids) {
        return receivableMapper.receivableList(ids);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByReceiptId(Long id) {
        return receivableMapper.getReceivableBackVoListByReceiptId(id);
    }

    @Override
    public ReceivableBackVO getReceivableBackVo(Long id) {
        return receivableMapper.getReceivableBackVo(id);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByOrderId(Long orderId) {
        return receivableMapper.getReceivableListByOrderId(orderId);
    }

    @Override
    public List<ReceivableBackVO> getFirstReceivableListByOrderId(Long orderId) {
        return receivableMapper.getFirstReceivableListByOrderId(orderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceivableDiscountById(ReceivableUpdateReqVO receivableUpdateReqVO) {
        ReceivableDO receivableDO = this.getReceivable(receivableUpdateReqVO.getId());
        if (null == receivableDO) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }

        if (null != receivableDO.getReceiptId()) {
            ReceiptDO receiptDO = receiptService.getReceipt(receivableDO.getReceiptId());
            if (null != receiptDO) {
                /*if (receiptDO.getState() == 4 || receiptDO.getState() == 5 || receiptDO.getState() == 6) {
                    throw exception(RECEIPT_STATUS_NOT_EXISTS);
                }*/
            }
        }

        ReceivableDO updateReceivableDO = new ReceivableDO();
        updateReceivableDO.setId(receivableDO.getId());
        updateReceivableDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        updateReceivableDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());
        this.updateById(updateReceivableDO);
        //存上应收款的优惠修改记录
        ReceivableDiscountDO receivableDiscountDO = new ReceivableDiscountDO();
        receivableDiscountDO.setReceivableId(receivableDO.getId());
        receivableDiscountDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        receivableDiscountDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());

        AdminUserRespDTO userRespDTO = adminUserApi.getUser(SecurityFrameworkUtils.getLoginUser().getId());
        if (null != userRespDTO) {
            receivableDiscountDO.setCreatorName(userRespDTO.getNickname());
        }

        receivableDiscountService.save(receivableDiscountDO);
    }

    @Override
    public List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(Long orderId) {

        return receivableMapper.getPayedMoneyByFeeType(orderId);
    }


    @Override
    public void orderReceivableAddReceiptLog(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收加入收款单", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public void orderReceivableWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            List<ReceivableDO> list = receivableMapper.selectList(new LambdaQueryWrapper<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
            // 如果没有找到订单应收，是不正常的，不需要去添加应收核销操作日志
            if (CollectionUtil.isNotEmpty(list)) {
                // 判断当前订单的应收是否有未核销完成的
                long count = list.stream().filter(r -> r.getState() != 2).count();
                if (count == 0) {
                    // 当前订单所有已生成的应收已经核销完成
                    log.error("-------------------------------");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收全部核销", Collections.singletonList(applyInfoVO)));
                } else {
                    log.error("222222222222222222222222222222222");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收部分核销", Collections.singletonList(applyInfoVO)));
                }
            }
        }
    }

    @Override
    public void orderReceivableCancelWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收反核销", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public List<ReceivableDO> receivableListByOrderId(Long orderId) {
        return receivableMapper.receivableListByOrderId(orderId);
    }


    @Override
    public List<ReceivableDO> allByOrderId(Long orderId) {
        return receivableMapper.allByOrderId(orderId);
    }

    @Override
    public List<ReceivableDO> getOrderPayList(Collection<Long> orderIdList) {
        return this.list(new LambdaQueryWrapper<ReceivableDO>()
                .ne(ReceivableDO::getState, 0)
                .eq(BaseDO::getDeleted, false)
                .in(ReceivableDO::getOrderId, orderIdList)
        );
    }

    @Override
    public void revokeDeleteReceivableById(Long receivableId) {
        receivableMapper.revokeDeleteReceivableById(receivableId);
    }
}
